// 
// Copyright (C) 2008 Christian Pointner, mur.at
//                    <equinox@spreadspace.org>
//
// This is free software, licensed under the GNU General Public License v2.
// See /LICENSE for more information.
//

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "menu.h"

cmdlist_entry* newCommandList()
{
  cmdlist_entry* ptr = malloc(sizeof(cmdlist_entry));
  if(!ptr)
    exit(2);

  ptr->command = NULL;
  ptr->next = NULL;

  return ptr;
}

void addCommand(cmdlist_entry** first, char* cmd)
{
  if(!first)
    return;

  if(!(*first)) {
    *first = newCommandList();
    (*first)->command = strdup(cmd);
    if(!(*first)->command)
      exit(2);

    return;
  }

  cmdlist_entry* current;
  for(current = *first; current; current = current->next) {
    if(!current->next) {
      current->next = newCommandList();
      current->next->command = strdup(cmd);
      if(!current->next->command)
        exit(2);

      return;
    }
  }
}

void clearCommandList(cmdlist_entry** first)
{
  cmdlist_entry* current;
  for(current = *first; current;) {
    cmdlist_entry* to_delete = current;
    current = current->next;

    if(to_delete->command)
      free(to_delete->command);
    free(to_delete);
  }
  *first = NULL;
}

void printCommandList(cmdlist_entry* first)
{
  if(!first) {
    printf("<empty>");
    return;
  }

  cmdlist_entry* current;
  for(current = first; current; current = current->next) {
    if(current->command)
      printf("'%s', ", current->command);
    else
      printf("<null>, ");
  }
}


menu_entry* newMenuEntry()
{
  menu_entry* ptr = malloc(sizeof(menu_entry));
  if(!ptr)
    exit(2);
  
  ptr->type = UNKNOWN;
  ptr->name = NULL;

  ptr->super_entry = NULL;
  ptr->sub_entry = NULL;
  ptr->cmd_list = NULL;

  ptr->prev = NULL;
  ptr->next = NULL;

  return ptr;
}

menu_entry* getFirst(menu_entry* entry)
{
  menu_entry* current;
  for(current = entry; current; current = current->prev)
    if(!current->prev) 
      return current;

  return NULL;
}

menu_entry* getLast(menu_entry* entry)
{
  menu_entry* current;
  for(current = entry; current; current = current->next)
    if(!current->next) 
      return current;

  return NULL;
}

menu_entry* addCommandList(menu_entry** first, char* name, cmdlist_entry* cmd_list)
{
  if(!first)
    return NULL;

  if(!(*first)) {
    *first = newMenuEntry();
    (*first)->name = strdup(name);
    if(!(*first)->name)
      exit(2);

    (*first)->cmd_list = cmd_list;
    (*first)->type = COMMAND;
    return *first;
  }
  
  menu_entry* new_ptr = newMenuEntry();
  new_ptr->name = strdup(name);
  if(!new_ptr->name)
    exit(2);

  new_ptr->cmd_list = cmd_list;
  new_ptr->type = COMMAND;

  menu_entry* last = getLast(*first);
  last->next = new_ptr;
  new_ptr->prev = last;

  return new_ptr;
}

menu_entry* addSubCommandList(menu_entry* super, char* name, cmdlist_entry* cmd_list)
{
  if(!super)
    return NULL;

  if(super->type != SUBMENU && super->type != NIL)
    return NULL;
  

  menu_entry* new_entry = addCommandList(&(super->sub_entry), name, cmd_list);
  if(super->sub_entry)
    super->sub_entry->super_entry = super;

  return new_entry;
}

menu_entry* addNamedEntry(menu_entry** first, char* name, entry_type type)
{
  if(!first)
    return NULL;

  if(!(*first)) {
    *first = newMenuEntry();
    (*first)->name = strdup(name);
    if(!(*first)->name)
      exit(2);

    (*first)->type = type;
    return *first;
  }
  
  menu_entry* new_ptr = newMenuEntry();
  new_ptr->name = strdup(name);
  if(!new_ptr->name)
    exit(2);

  new_ptr->type = type;

  menu_entry* last = getLast(*first);
  last->next = new_ptr;
  new_ptr->prev = last;

  return new_ptr;
}

menu_entry* addSubNamedEntry(menu_entry* super, char* name, entry_type type)
{
  if(!super)
    return NULL;

  if(super->type != SUBMENU && super->type != NIL)
    return NULL;
  
  menu_entry* new_entry = addNamedEntry(&(super->sub_entry), name, type);
  if(super->sub_entry)
    super->sub_entry->super_entry = super;

  return new_entry;
}

menu_entry* addShowCommand(menu_entry** first, char* name, char* cmd)
{
  if(!first)
    return NULL;

  if(!(*first)) {
    *first = newMenuEntry();
    (*first)->name = strdup(name);
    if(!(*first)->name)
      exit(2);

    (*first)->show_cmd = strdup(cmd);
    if(!(*first)->show_cmd)
      exit(2);

    (*first)->type = SHOW;
    return *first;
  }
  
  menu_entry* new_ptr = newMenuEntry();
  new_ptr->name = strdup(name);
  if(!new_ptr->name)
    exit(2);

  new_ptr->show_cmd = strdup(cmd);
  if(!new_ptr->show_cmd)
    exit(2);

  new_ptr->type = SHOW;

  menu_entry* last = getLast(*first);
  last->next = new_ptr;
  new_ptr->prev = last;

  return new_ptr;
}

menu_entry* addSubShowCommand(menu_entry* super, char* name, char* cmd)
{
  if(!super)
    return NULL;

  if(super->type != SUBMENU && super->type != NIL)
    return NULL;
  
  menu_entry* new_entry = addShowCommand(&(super->sub_entry), name, cmd);
  if(super->sub_entry)
    super->sub_entry->super_entry = super;

  return new_entry;
}

menu_entry* getSubEntry(menu_entry* entry)
{
  if(entry) 
    return entry->sub_entry;

  return NULL;
}

menu_entry* getSuperEntry(menu_entry* entry)
{
  menu_entry* first = getFirst(entry);
  if(first)
    return first->super_entry;
  
  return NULL;
}

void clearMenu(menu_entry** first)
{
  menu_entry* current;
  for(current = *first; current;) {
    menu_entry* to_delete = current;
    current = current->next;

    if(to_delete->name)
      free(to_delete->name);

    if(to_delete->type == SUBMENU || to_delete->type == NIL)
      clearMenu(&(to_delete->sub_entry));
    else if(to_delete->type == COMMAND)
      clearCommandList(&(to_delete->cmd_list));
    else if(to_delete->type == SHOW && to_delete->show_cmd)
      free(to_delete->show_cmd);

    free(to_delete);
  }
  *first = NULL;
}

void printSpaces(int cnt)
{
  printf("+");
  for(;cnt;cnt--)
    printf("-");
}

void printMenu(menu_entry* first, int level)
{
  if(!first && !level) {
    printf("<empty>\n");
    return;
  }

  menu_entry* current;
  for(current = first; current; current = current->next) {
    if(current->name) {
      printSpaces(level);
      printf("%s", current->name);
    }
    else {
      printSpaces(level);
      printf("<null>");
    }

    if(current->type == SUBMENU || current->type == NIL) {
      printf("\n");
      printMenu(current->sub_entry, level+1);
    }
    else if(current->type == COMMAND) {
      printf(" -> ");
      printCommandList(current->cmd_list);
      printf("\n");
    }
    else if(current->type == SHOW) {
      printf(" -> '%s'\n", current->show_cmd);
    }
    else if(current->type == DISPLAY)
      printf("\n");
  }
}

